/* SPDX-License-Identifier: GPL-2.0 + WITH Linux-syscall-note */
/*
 * Copyright (c) 2023-24 Raspberry Pi Ltd.
 * All rights reserved.
 */
#ifndef _PIO_RP1_IF_H
#define _PIO_RP1_IF_H

#include <linux/types.h>
#include <linux/ioctl.h>

#define RP1_PIO_INSTRUCTION_COUNT   32
#define RP1_PIO_SM_COUNT            4
#define RP1_PIO_GPIO_COUNT          28
#define RP1_GPIO_FUNC_PIO           7

#define RP1_PIO_ORIGIN_ANY          ((__u16)(~0))

#define RP1_PIO_DIR_TO_SM           0
#define RP1_PIO_DIR_FROM_SM         1
#define RP1_PIO_DIR_COUNT           2

typedef struct {
	__u32 clkdiv;
	__u32 execctrl;
	__u32 shiftctrl;
	__u32 pinctrl;
} rp1_pio_sm_config;

struct rp1_pio_add_program_args {
	__u16 num_instrs;
	__u16 origin;
	__u16 instrs[RP1_PIO_INSTRUCTION_COUNT];
};

struct rp1_pio_remove_program_args {
	__u16 num_instrs;
	__u16 origin;
};

struct rp1_pio_sm_claim_args {
	__u16 mask;
};

struct rp1_pio_sm_init_args {
	__u16 sm;
	__u16 initial_pc;
	rp1_pio_sm_config config;
};

struct rp1_pio_sm_set_config_args {
	__u16 sm;
	__u16 rsvd;
	rp1_pio_sm_config config;
};

struct rp1_pio_sm_exec_args {
	__u16 sm;
	__u16 instr;
	__u8 blocking;
	__u8 rsvd;
};

struct rp1_pio_sm_clear_fifos_args {
	__u16 sm;
};

struct rp1_pio_sm_set_clkdiv_args {
	__u16 sm;
	__u16 div_int;
	__u8 div_frac;
	__u8 rsvd;
};

struct rp1_pio_sm_set_pins_args {
	__u16 sm;
	__u16 rsvd;
	__u32 values;
	__u32 mask;
};

struct rp1_pio_sm_set_pindirs_args {
	__u16 sm;
	__u16 rsvd;
	__u32 dirs;
	__u32 mask;
};

struct rp1_pio_sm_set_enabled_args {
	__u16 mask;
	__u8 enable;
	__u8 rsvd;
};

struct rp1_pio_sm_restart_args {
	__u16 mask;
};

struct rp1_pio_sm_clkdiv_restart_args {
	__u16 mask;
};

struct rp1_pio_sm_enable_sync_args {
	__u16 mask;
};

struct rp1_pio_sm_put_args {
	__u16 sm;
	__u8 blocking;
	__u8 rsvd;
	__u32 data;
};

struct rp1_pio_sm_get_args {
	__u16 sm;
	__u8 blocking;
	__u8 rsvd;
	__u32 data; /* OUT */
};

struct rp1_pio_sm_set_dmactrl_args {
	__u16 sm;
	__u8 is_tx;
	__u8 rsvd;
	__u32 ctrl;
};

struct rp1_pio_sm_fifo_state_args {
	__u16 sm;
	__u8 tx;
	__u8 rsvd;
	__u16 level; /* OUT */
	__u8 empty; /* OUT */
	__u8 full; /* OUT */
};

struct rp1_gpio_init_args {
	__u16 gpio;
};

struct rp1_gpio_set_function_args {
	__u16 gpio;
	__u16 fn;
};

struct rp1_gpio_set_pulls_args {
	__u16 gpio;
	__u8 up;
	__u8 down;
};

struct rp1_gpio_set_args {
	__u16 gpio;
	__u16 value;
};

struct rp1_pio_sm_config_xfer_args {
	__u16 sm;
	__u16 dir;
	__u16 buf_size;
	__u16 buf_count;
};

struct rp1_pio_sm_config_xfer32_args {
	__u16 sm;
	__u16 dir;
	__u32 buf_size;
	__u32 buf_count;
};

struct rp1_pio_sm_xfer_data_args {
	__u16 sm;
	__u16 dir;
	__u16 data_bytes;
	void *data;
};

struct rp1_pio_sm_xfer_data32_args {
	__u16 sm;
	__u16 dir;
	__u32 data_bytes;
	void *data;
};

struct rp1_access_hw_args {
	__u32 addr;
	__u32 len;
	void *data;
};

#define PIO_IOC_MAGIC 102

#define PIO_IOC_SM_CONFIG_XFER _IOW(PIO_IOC_MAGIC, 0, struct rp1_pio_sm_config_xfer_args)
#define PIO_IOC_SM_XFER_DATA _IOW(PIO_IOC_MAGIC, 1, struct rp1_pio_sm_xfer_data_args)
#define PIO_IOC_SM_XFER_DATA32 _IOW(PIO_IOC_MAGIC, 2, struct rp1_pio_sm_xfer_data32_args)
#define PIO_IOC_SM_CONFIG_XFER32 _IOW(PIO_IOC_MAGIC, 3, struct rp1_pio_sm_config_xfer32_args)

#define PIO_IOC_READ_HW _IOW(PIO_IOC_MAGIC, 8, struct rp1_access_hw_args)
#define PIO_IOC_WRITE_HW _IOW(PIO_IOC_MAGIC, 9, struct rp1_access_hw_args)

#define PIO_IOC_CAN_ADD_PROGRAM _IOW(PIO_IOC_MAGIC, 10, struct rp1_pio_add_program_args)
#define PIO_IOC_ADD_PROGRAM _IOW(PIO_IOC_MAGIC, 11, struct rp1_pio_add_program_args)
#define PIO_IOC_REMOVE_PROGRAM _IOW(PIO_IOC_MAGIC, 12, struct rp1_pio_remove_program_args)
#define PIO_IOC_CLEAR_INSTR_MEM _IO(PIO_IOC_MAGIC, 13)

#define PIO_IOC_SM_CLAIM _IOW(PIO_IOC_MAGIC, 20, struct rp1_pio_sm_claim_args)
#define PIO_IOC_SM_UNCLAIM _IOW(PIO_IOC_MAGIC, 21, struct rp1_pio_sm_claim_args)
#define PIO_IOC_SM_IS_CLAIMED _IOW(PIO_IOC_MAGIC, 22, struct rp1_pio_sm_claim_args)

#define PIO_IOC_SM_INIT _IOW(PIO_IOC_MAGIC, 30, struct rp1_pio_sm_init_args)
#define PIO_IOC_SM_SET_CONFIG _IOW(PIO_IOC_MAGIC, 31, struct rp1_pio_sm_set_config_args)
#define PIO_IOC_SM_EXEC _IOW(PIO_IOC_MAGIC, 32, struct rp1_pio_sm_exec_args)
#define PIO_IOC_SM_CLEAR_FIFOS _IOW(PIO_IOC_MAGIC, 33, struct rp1_pio_sm_clear_fifos_args)
#define PIO_IOC_SM_SET_CLKDIV _IOW(PIO_IOC_MAGIC, 34, struct rp1_pio_sm_set_clkdiv_args)
#define PIO_IOC_SM_SET_PINS _IOW(PIO_IOC_MAGIC, 35, struct rp1_pio_sm_set_pins_args)
#define PIO_IOC_SM_SET_PINDIRS _IOW(PIO_IOC_MAGIC, 36, struct rp1_pio_sm_set_pindirs_args)
#define PIO_IOC_SM_SET_ENABLED _IOW(PIO_IOC_MAGIC, 37, struct rp1_pio_sm_set_enabled_args)
#define PIO_IOC_SM_RESTART _IOW(PIO_IOC_MAGIC, 38, struct rp1_pio_sm_restart_args)
#define PIO_IOC_SM_CLKDIV_RESTART _IOW(PIO_IOC_MAGIC, 39, struct rp1_pio_sm_restart_args)
#define PIO_IOC_SM_ENABLE_SYNC _IOW(PIO_IOC_MAGIC, 40, struct rp1_pio_sm_enable_sync_args)
#define PIO_IOC_SM_PUT _IOW(PIO_IOC_MAGIC, 41, struct rp1_pio_sm_put_args)
#define PIO_IOC_SM_GET _IOWR(PIO_IOC_MAGIC, 42, struct rp1_pio_sm_get_args)
#define PIO_IOC_SM_SET_DMACTRL _IOW(PIO_IOC_MAGIC, 43, struct rp1_pio_sm_set_dmactrl_args)
#define PIO_IOC_SM_FIFO_STATE _IOW(PIO_IOC_MAGIC, 44, struct rp1_pio_sm_fifo_state_args)
#define PIO_IOC_SM_DRAIN_TX _IOW(PIO_IOC_MAGIC, 45, struct rp1_pio_sm_clear_fifos_args)

#define PIO_IOC_GPIO_INIT _IOW(PIO_IOC_MAGIC, 50, struct rp1_gpio_init_args)
#define PIO_IOC_GPIO_SET_FUNCTION _IOW(PIO_IOC_MAGIC, 51, struct rp1_gpio_set_function_args)
#define PIO_IOC_GPIO_SET_PULLS _IOW(PIO_IOC_MAGIC, 52, struct rp1_gpio_set_pulls_args)
#define PIO_IOC_GPIO_SET_OUTOVER _IOW(PIO_IOC_MAGIC, 53, struct rp1_gpio_set_args)
#define PIO_IOC_GPIO_SET_INOVER _IOW(PIO_IOC_MAGIC, 54, struct rp1_gpio_set_args)
#define PIO_IOC_GPIO_SET_OEOVER _IOW(PIO_IOC_MAGIC, 55, struct rp1_gpio_set_args)
#define PIO_IOC_GPIO_SET_INPUT_ENABLED _IOW(PIO_IOC_MAGIC, 56, struct rp1_gpio_set_args)
#define PIO_IOC_GPIO_SET_DRIVE_STRENGTH _IOW(PIO_IOC_MAGIC, 57, struct rp1_gpio_set_args)

#endif
